/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001-2004 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.ui.jsp;

import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.ui.*;
import com.silverwrist.venice.ui.helpers.HTMLRendering;

public class UtilXLinkTag extends VeniceBodyTagSupport
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static final String LOCATION_PARAM = "${location}";

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String href = null;
  private int type = -1;
  private String cls = null;
  private String target = null;
  private String text = null;
  private String title = null;
  private boolean target_external = false;

  /*--------------------------------------------------------------------------------
   * Overrides from class BodyTagSupport
   *--------------------------------------------------------------------------------
   */

  public int doStartTag()
  {
    return EVAL_BODY_BUFFERED;

  } // end doStartTag

  public int doAfterBody()
  {
    return SKIP_BODY;

  } // end doAfterBody

  public int doEndTag() throws JspException
  {
    // Do some shorts-checking on our stored data.
    if (href==null)
      throw new JspTagException("<util:xlink/> does not have a <util:href/>!");
    if (type<0)
      throw new JspTagException("<util:xlink/> does not have a <util:href/>!");
    if (text==null)
      throw new JspTagException("<util:xlink/> does not have a <util:text/>!");

    // Dig out the RequestInput and HTMLRendering service.
    RequestInput ri = getRequestInput();
    HTMLRendering html = (HTMLRendering)(ri.queryService(HTMLRendering.class));

    try
    { // Do parameter replacement on the HREF string if necessary.
      if (href.indexOf(LOCATION_PARAM)>=0)
	href = StringUtil.replaceAllInstances(href,LOCATION_PARAM,URLEncoder.encode(ri.getLocation(),"UTF-8"));

      // write out what we came here to accomplish
      JspWriter out = pageContext.getOut();
      out.write("<A ");
      if (cls!=null)
	out.write("CLASS=\"" + cls + "\" ");
      out.write("HREF=\"" + html.formatURL(href,type) + "\"");
      if (title!=null)
	out.write(" TITLE=\"" + title + "\"");
      if (target!=null)
	out.write(" TARGET=\"" + target + "\"");
      out.write(">" + text + "</A>");

    } // end try
    catch (IOException e)
    { // convert the I/O error into something the servlet engine can deal with
      throw new JspTagException("error writing <A> tag - " + e.getMessage());

    } // end catch

    href = null;
    type = -1;
    text = null;
    title = null;
    if (target_external)
      target = null;
    target_external = false;
    return EVAL_PAGE;

  } // end doEndTag

  public void release()
  {
    super.release();
    cls = null;
    target = null;

  } // end release

  /*--------------------------------------------------------------------------------
   * Attribute set functions
   *--------------------------------------------------------------------------------
   */

  public void setAclass(String s)
  {
    cls = s;

  } // end setAclass

  public void setTarget(String s)
  {
    target = s;

  } // end setTarget

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  final void setLink(String href, int type)
  {
    this.href = href;
    this.type = type;

  } // end setLink

  final void setText(String text)
  {
    this.text = text;

  } // end setText

  final void setTitle(String title)
  {
    this.title = title;

  } // end setTitle

  final void setTarget2(String s)
  {
    if (target==null)
    { // set target via external tag
      target = s;
      target_external = true;

    } // end if

  } // end setTarget2

} // end class UtilXLinkTag
